@@ -149,11 +149,11 @@ fn main() {
                         break 'read;
                     }
 
-                    panic!("recv() failed: {:?}", e);
+                    panic!("recv() failed: {e:?}");
                 },
             };
 
-            debug!("got {} bytes", len);
+            debug!("got {len} bytes");
 
             let pkt_buf = &mut buf[..len];
 
@@ -165,12 +165,12 @@ fn main() {
                 Ok(v) => v,
 
                 Err(e) => {
-                    error!("Parsing packet header failed: {:?}", e);
+                    error!("Parsing packet header failed: {e:?}");
                     continue 'read;
                 },
             };
 
-            trace!("got packet {:?}", hdr);
+            trace!("got packet {hdr:?}");
 
             let conn_id = ring::hmac::sign(&conn_id_seed, &hdr.dcid);
             let conn_id = &conn_id.as_ref()[..quiche::MAX_CONN_ID_LEN];
@@ -201,7 +201,7 @@ fn main() {
                             break;
                         }
 
-                        panic!("send() failed: {:?}", e);
+                        panic!("send() failed: {e:?}");
                     }
                     continue 'read;
                 }
@@ -238,7 +238,7 @@ fn main() {
                             break;
                         }
 
-                        panic!("send() failed: {:?}", e);
+                        panic!("send() failed: {e:?}");
                     }
                     continue 'read;
                 }
@@ -323,7 +323,7 @@ fn main() {
                     Ok(v) => v,
 
                     Err(e) => {
-                        error!("failed to create HTTP/3 connection: {}", e);
+                        error!("failed to create HTTP/3 connection: {e}");
                         continue 'read;
                     },
                 };
@@ -419,7 +419,7 @@ fn main() {
                         break;
                     }
 
-                    panic!("send() failed: {:?}", e);
+                    panic!("send() failed: {e:?}");
                 }
 
                 debug!("{} written {} bytes", client.conn.trace_id(), write);
