@@ -24,16 +24,19 @@
 // NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 // SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-use std::time;
-
 use std::collections::BTreeMap;
 use std::collections::VecDeque;
+
 use std::net::SocketAddr;
 
+use std::time::Duration;
+use std::time::Instant;
+
 use smallvec::SmallVec;
 
 use slab::Slab;
 
+use crate::Config;
 use crate::Error;
 use crate::Result;
 use crate::StartupExit;
@@ -139,12 +142,12 @@ pub struct Path {
     /// Loss recovery and congestion control state.
     pub recovery: recovery::Recovery,
 
-    /// Path MTU discovery state.
-    pub pmtud: pmtud::Pmtud,
+    /// Path MTU discovery state. None if PMTUD is disabled on the path.
+    pub pmtud: Option<pmtud::Pmtud>,
 
     /// Pending challenge data with the size of the packet containing them and
     /// when they were sent.
-    in_flight_challenges: VecDeque<([u8; 8], usize, time::Instant)>,
+    in_flight_challenges: VecDeque<([u8; 8], usize, Instant)>,
 
     /// The maximum challenge size that got acknowledged.
     max_challenge_size: usize,
@@ -153,7 +156,7 @@ pub struct Path {
     probing_lost: usize,
 
     /// Last instant when a probing packet got lost.
-    last_probe_lost_time: Option<time::Instant>,
+    last_probe_lost_time: Option<Instant>,
 
     /// Received challenge data.
     received_challenges: VecDeque<[u8; 8]>,
@@ -224,8 +227,8 @@ impl Path {
     pub fn new(
         local_addr: SocketAddr, peer_addr: SocketAddr,
         recovery_config: &recovery::RecoveryConfig,
-        path_challenge_recv_max_queue_len: usize, pmtud_init: usize,
-        is_initial: bool,
+        path_challenge_recv_max_queue_len: usize, is_initial: bool,
+        config: Option<&Config>,
     ) -> Self {
         let (state, active_scid_seq, active_dcid_seq) = if is_initial {
             (PathState::Validated, Some(0), Some(0))
@@ -233,6 +236,23 @@ impl Path {
             (PathState::Unknown, None, None)
         };
 
+        let pmtud = config.and_then(|c| {
+            if c.pmtud {
+                let maximum_supported_mtu: usize = std::cmp::min(
+                    // if the max_udp_payload_size doesn't fit into a usize, then
+                    // max_send_udp_payload_size must be smaller so use that
+                    c.local_transport_params
+                        .max_udp_payload_size
+                        .try_into()
+                        .unwrap_or(c.max_send_udp_payload_size),
+                    c.max_send_udp_payload_size,
+                );
+                Some(pmtud::Pmtud::new(maximum_supported_mtu))
+            } else {
+                None
+            }
+        });
+
         Self {
             local_addr,
             peer_addr,
@@ -241,7 +261,7 @@ impl Path {
             state,
             active: false,
             recovery: recovery::Recovery::new_with_config(recovery_config),
-            pmtud: pmtud::Pmtud::new(pmtud_init),
+            pmtud,
             in_flight_challenges: VecDeque::new(),
             max_challenge_size: 0,
             probing_lost: 0,
@@ -356,11 +376,15 @@ impl Path {
         &mut self, hs_confirmed: bool, hs_done: bool, out_len: usize,
         is_closing: bool, frames_empty: bool,
     ) -> bool {
+        let Some(pmtud) = self.pmtud.as_mut() else {
+            return false;
+        };
+
         (hs_confirmed && hs_done) &&
-            self.pmtud.get_probe_size() > self.pmtud.get_current_mtu() &&
-            self.recovery.cwnd_available() > self.pmtud.get_probe_size() &&
-            out_len >= self.pmtud.get_probe_size() &&
-            self.pmtud.get_should_probe() &&
+            pmtud.get_probe_size() > pmtud.get_current_mtu() &&
+            self.recovery.cwnd_available() > pmtud.get_probe_size() &&
+            out_len >= pmtud.get_probe_size() &&
+            pmtud.should_probe() &&
             !is_closing &&
             frames_empty
     }
@@ -372,7 +396,7 @@ impl Path {
 
     /// Handles the sending of PATH_CHALLENGE.
     pub fn add_challenge_sent(
-        &mut self, data: [u8; 8], pkt_size: usize, sent_time: time::Instant,
+        &mut self, data: [u8; 8], pkt_size: usize, sent_time: Instant,
     ) {
         self.on_challenge_sent();
         self.in_flight_challenges
@@ -439,7 +463,7 @@ impl Path {
     }
 
     pub fn on_loss_detection_timeout(
-        &mut self, handshake_status: HandshakeStatus, now: time::Instant,
+        &mut self, handshake_status: HandshakeStatus, now: Instant,
         is_server: bool, trace_id: &str,
     ) -> OnLossDetectionTimeoutOutcome {
         let outcome = self.recovery.on_loss_detection_timeout(
@@ -530,7 +554,7 @@ impl Path {
         }
     }
 
-    pub fn bytes_in_flight_duration(&self) -> time::Duration {
+    pub fn bytes_in_flight_duration(&self) -> Duration {
         self.recovery.bytes_in_flight_duration()
     }
 }
@@ -585,7 +609,6 @@ impl PathMap {
     /// capacity limit.
     pub fn new(
         mut initial_path: Path, max_concurrent_paths: usize, is_server: bool,
-        enable_pmtud: bool, max_send_udp_payload_size: usize,
     ) -> Self {
         let mut paths = Slab::with_capacity(1); // most connections only have one path
         let mut addrs_to_paths = BTreeMap::new();
@@ -596,14 +619,6 @@ impl PathMap {
         // As it is the first path, it is active by default.
         initial_path.active = true;
 
-        // Enable path MTU Discovery and start probing with the largest datagram
-        // size.
-        if enable_pmtud {
-            initial_path.pmtud.set_should_probe(enable_pmtud);
-            initial_path.pmtud.set_probe_size(max_send_udp_payload_size);
-            initial_path.pmtud.enable(enable_pmtud);
-        }
-
         let active_path_id = paths.insert(initial_path);
         addrs_to_paths.insert((local_addr, peer_addr), active_path_id);
 
@@ -866,11 +881,11 @@ impl PathMap {
         &mut self, discover: bool, max_send_udp_payload_size: usize,
     ) {
         for (_, path) in self.paths.iter_mut() {
-            path.pmtud.enable(discover);
-            // It's harmless to modify probe size even if we're disabling
-            // pmtud.
-            path.pmtud.set_probe_size(max_send_udp_payload_size);
-            path.pmtud.set_should_probe(discover);
+            path.pmtud = if discover {
+                Some(pmtud::Pmtud::new(max_send_udp_payload_size))
+            } else {
+                None
+            };
         }
     }
 }
@@ -922,17 +937,17 @@ pub struct PathStats {
     pub dgram_sent: usize,
 
     /// The estimated round-trip time of the connection.
-    pub rtt: time::Duration,
+    pub rtt: Duration,
 
     /// The minimum round-trip time observed.
-    pub min_rtt: Option<time::Duration>,
+    pub min_rtt: Option<Duration>,
 
     /// The maximum round-trip time observed.
-    pub max_rtt: Option<time::Duration>,
+    pub max_rtt: Option<Duration>,
 
     /// The estimated round-trip time variation in samples using a mean
     /// variation.
-    pub rttvar: time::Duration,
+    pub rttvar: Duration,
 
     /// The size of the connection's congestion window in bytes.
     pub cwnd: usize,
@@ -1023,18 +1038,18 @@ mod tests {
             server_addr,
             &recovery_config,
             config.path_challenge_recv_max_queue_len,
-            1200,
             true,
+            None,
         );
-        let mut path_mgr = PathMap::new(path, 2, false, true, 1200);
+        let mut path_mgr = PathMap::new(path, 2, false);
 
         let probed_path = Path::new(
             client_addr_2,
             server_addr,
             &recovery_config,
             config.path_challenge_recv_max_queue_len,
-            1200,
             false,
+            None,
         );
         path_mgr.insert_path(probed_path, false).unwrap();
 
@@ -1051,7 +1066,7 @@ mod tests {
         path_mgr.get_mut(pid).unwrap().add_challenge_sent(
             data,
             MIN_CLIENT_INITIAL_LEN - 1,
-            time::Instant::now(),
+            Instant::now(),
         );
 
         assert!(!path_mgr.get_mut(pid).unwrap().validation_requested());
@@ -1081,7 +1096,7 @@ mod tests {
         path_mgr.get_mut(pid).unwrap().add_challenge_sent(
             data,
             MIN_CLIENT_INITIAL_LEN,
-            time::Instant::now(),
+            Instant::now(),
         );
 
         path_mgr.on_response_received(data).unwrap();
@@ -1110,17 +1125,17 @@ mod tests {
             server_addr,
             &recovery_config,
             config.path_challenge_recv_max_queue_len,
-            1200,
             true,
+            None,
         );
-        let mut client_path_mgr = PathMap::new(path, 2, false, false, 1200);
+        let mut client_path_mgr = PathMap::new(path, 2, false);
         let mut server_path = Path::new(
             server_addr,
             client_addr,
             &recovery_config,
             config.path_challenge_recv_max_queue_len,
-            1200,
             false,
+            None,
         );
 
         let client_pid = client_path_mgr
@@ -1133,11 +1148,7 @@ mod tests {
         client_path_mgr
             .get_mut(client_pid)
             .unwrap()
-            .add_challenge_sent(
-                data,
-                MIN_CLIENT_INITIAL_LEN,
-                time::Instant::now(),
-            );
+            .add_challenge_sent(data, MIN_CLIENT_INITIAL_LEN, Instant::now());
 
         // Second probe.
         let data_2 = rand::rand_u64().to_be_bytes();
@@ -1145,11 +1156,7 @@ mod tests {
         client_path_mgr
             .get_mut(client_pid)
             .unwrap()
-            .add_challenge_sent(
-                data_2,
-                MIN_CLIENT_INITIAL_LEN,
-                time::Instant::now(),
-            );
+            .add_challenge_sent(data_2, MIN_CLIENT_INITIAL_LEN, Instant::now());
         assert_eq!(
             client_path_mgr
                 .get(client_pid)
@@ -1202,17 +1209,17 @@ mod tests {
             server_addr,
             &recovery_config,
             config.path_challenge_recv_max_queue_len,
-            1200,
             true,
+            None,
         );
-        let mut client_path_mgr = PathMap::new(path, 2, false, false, 1200);
+        let mut client_path_mgr = PathMap::new(path, 2, false);
         let mut server_path = Path::new(
             server_addr,
             client_addr,
             &recovery_config,
             config.path_challenge_recv_max_queue_len,
-            1200,
             false,
+            None,
         );
 
         let client_pid = client_path_mgr
@@ -1225,11 +1232,7 @@ mod tests {
         client_path_mgr
             .get_mut(client_pid)
             .unwrap()
-            .add_challenge_sent(
-                data,
-                MIN_CLIENT_INITIAL_LEN,
-                time::Instant::now(),
-            );
+            .add_challenge_sent(data, MIN_CLIENT_INITIAL_LEN, Instant::now());
 
         // Second probe.
         let data_2 = rand::rand_u64().to_be_bytes();
@@ -1237,11 +1240,7 @@ mod tests {
         client_path_mgr
             .get_mut(client_pid)
             .unwrap()
-            .add_challenge_sent(
-                data_2,
-                MIN_CLIENT_INITIAL_LEN,
-                time::Instant::now(),
-            );
+            .add_challenge_sent(data_2, MIN_CLIENT_INITIAL_LEN, Instant::now());
         assert_eq!(
             client_path_mgr
                 .get(client_pid)
@@ -1257,11 +1256,7 @@ mod tests {
         client_path_mgr
             .get_mut(client_pid)
             .unwrap()
-            .add_challenge_sent(
-                data_3,
-                MIN_CLIENT_INITIAL_LEN,
-                time::Instant::now(),
-            );
+            .add_challenge_sent(data_3, MIN_CLIENT_INITIAL_LEN, Instant::now());
         assert_eq!(
             client_path_mgr
                 .get(client_pid)
@@ -1277,11 +1272,7 @@ mod tests {
         client_path_mgr
             .get_mut(client_pid)
             .unwrap()
-            .add_challenge_sent(
-                data_4,
-                MIN_CLIENT_INITIAL_LEN,
-                time::Instant::now(),
-            );
+            .add_challenge_sent(data_4, MIN_CLIENT_INITIAL_LEN, Instant::now());
         assert_eq!(
             client_path_mgr
                 .get(client_pid)
