@@ -38,10 +38,9 @@ use crate::recovery::MAX_PTO_PROBES_COUNT;
 use crate::Error;
 use crate::Result;
 
+use super::bbr2::BBRv2;
 use super::pacer::Pacer;
 use super::Acked;
-use super::Congestion;
-use super::CongestionControl;
 use super::Lost;
 
 // Congestion Control
@@ -128,7 +127,7 @@ struct LossDetectionResult {
 impl RecoveryEpoch {
     /// Discard the Epoch state and return the total size of unacked packets
     /// that were discarded
-    fn discard(&mut self, cc: &mut impl CongestionControl) -> usize {
+    fn discard(&mut self, cc: &mut Pacer) -> usize {
         let unacked_bytes = self
             .sent_packets
             .drain(..)
@@ -230,7 +229,7 @@ impl RecoveryEpoch {
 
                             has_ack_eliciting |= ack_eliciting;
 
-                            trace!("{} packet newly acked {}", trace_id, pkt_num);
+                            trace!("{trace_id} packet newly acked {pkt_num}");
                         },
 
                         SentStatus::Acked => {},
@@ -381,6 +380,9 @@ pub struct GRecovery {
     #[cfg(feature = "qlog")]
     qlog_metrics: QlogMetrics,
 
+    #[cfg(feature = "qlog")]
+    qlog_prev_cc_state: &'static str,
+
     /// How many non-ack-eliciting packets have been sent.
     outstanding_non_ack_eliciting: usize,
 
@@ -397,17 +399,22 @@ pub struct GRecovery {
 impl GRecovery {
     pub fn new(recovery_config: &RecoveryConfig) -> Option<Self> {
         let cc = match recovery_config.cc_algorithm {
-            CongestionControlAlgorithm::Bbr2Gcongestion => Congestion::bbrv2(
+            CongestionControlAlgorithm::Bbr2Gcongestion => BBRv2::new(
                 recovery_config.initial_congestion_window_packets,
                 MAX_WINDOW_PACKETS,
-                recovery_config,
+                recovery_config.max_send_udp_payload_size,
+                recovery_config.initial_rtt,
+                recovery_config.custom_bbr_params.as_ref(),
             ),
             _ => return None,
         };
 
         Some(Self {
             epochs: Default::default(),
-            rtt_stats: RttStats::new(recovery_config.max_ack_delay),
+            rtt_stats: RttStats::new(
+                recovery_config.initial_rtt,
+                recovery_config.max_ack_delay,
+            ),
             recovery_stats: RecoveryStats::default(),
             loss_timer: Default::default(),
             pto_count: 0,
@@ -427,6 +434,9 @@ impl GRecovery {
             #[cfg(feature = "qlog")]
             qlog_metrics: QlogMetrics::default(),
 
+            #[cfg(feature = "qlog")]
+            qlog_prev_cc_state: "",
+
             outstanding_non_ack_eliciting: 0,
 
             pacer: Pacer::new(
@@ -665,7 +675,7 @@ impl RecoveryOps for GRecovery {
 
         self.bytes_sent += sent_bytes;
 
-        trace!("{} {:?}", trace_id, self);
+        trace!("{trace_id} {self:?}");
     }
 
     fn get_packet_send_time(&self, now: Instant) -> Instant {
@@ -747,7 +757,7 @@ impl RecoveryOps for GRecovery {
 
         self.set_loss_detection_timer(handshake_status, now);
 
-        trace!("{} {:?}", trace_id, self);
+        trace!("{trace_id} {self:?}");
 
         Ok(OnAckReceivedOutcome {
             lost_packets,
@@ -785,7 +795,7 @@ impl RecoveryOps for GRecovery {
 
             self.set_loss_detection_timer(handshake_status, now);
 
-            trace!("{} {:?}", trace_id, self);
+            trace!("{trace_id} {self:?}");
             return OnLossDetectionTimeoutOutcome {
                 lost_packets,
                 lost_bytes,
@@ -851,7 +861,7 @@ impl RecoveryOps for GRecovery {
 
         self.set_loss_detection_timer(handshake_status, now);
 
-        trace!("{} {:?}", trace_id, self);
+        trace!("{trace_id} {self:?}");
         OnLossDetectionTimeoutOutcome {
             lost_packets: 0,
             lost_bytes: 0,
@@ -1028,7 +1038,12 @@ impl RecoveryOps for GRecovery {
     }
 
     #[cfg(feature = "qlog")]
-    fn maybe_qlog(&mut self) -> Option<EventData> {
+    fn state_str(&self, _now: Instant) -> &'static str {
+        self.pacer.state_str()
+    }
+
+    #[cfg(feature = "qlog")]
+    fn get_updated_qlog_event_data(&mut self) -> Option<EventData> {
         let qlog_metrics = QlogMetrics {
             min_rtt: *self.rtt_stats.min_rtt,
             smoothed_rtt: self.rtt(),
@@ -1043,6 +1058,19 @@ impl RecoveryOps for GRecovery {
         self.qlog_metrics.maybe_update(qlog_metrics)
     }
 
+    #[cfg(feature = "qlog")]
+    fn get_updated_qlog_cc_state(
+        &mut self, now: Instant,
+    ) -> Option<&'static str> {
+        let cc_state = self.state_str(now);
+        if cc_state != self.qlog_prev_cc_state {
+            self.qlog_prev_cc_state = cc_state;
+            Some(cc_state)
+        } else {
+            None
+        }
+    }
+
     fn send_quantum(&self) -> usize {
         let pacing_rate = self
             .pacer
