@@ -98,6 +98,7 @@ impl std::fmt::Debug for LossDetectionTimer {
 
 #[derive(Clone, Copy, PartialEq)]
 pub struct RecoveryConfig {
+    pub initial_rtt: Duration,
     pub max_send_udp_payload_size: usize,
     pub max_ack_delay: Duration,
     pub cc_algorithm: CongestionControlAlgorithm,
@@ -111,6 +112,7 @@ pub struct RecoveryConfig {
 impl RecoveryConfig {
     pub fn from_config(config: &Config) -> Self {
         Self {
+            initial_rtt: config.initial_rtt,
             max_send_udp_payload_size: config.max_send_udp_payload_size,
             max_ack_delay: Duration::ZERO,
             cc_algorithm: config.cc_algorithm,
@@ -174,6 +176,7 @@ pub trait RecoveryOps {
     );
     fn get_packet_send_time(&self, now: Instant) -> Instant;
 
+    #[allow(clippy::too_many_arguments)]
     fn on_ack_received(
         &mut self, ranges: &RangeSet, ack_delay: u64, epoch: packet::Epoch,
         handshake_status: HandshakeStatus, now: Instant, skip_pn: Option<u64>,
@@ -261,7 +264,15 @@ pub trait RecoveryOps {
     fn update_max_ack_delay(&mut self, max_ack_delay: Duration);
 
     #[cfg(feature = "qlog")]
-    fn maybe_qlog(&mut self) -> Option<EventData>;
+    fn state_str(&self, now: Instant) -> &'static str;
+
+    #[cfg(feature = "qlog")]
+    fn get_updated_qlog_event_data(&mut self) -> Option<EventData>;
+
+    #[cfg(feature = "qlog")]
+    fn get_updated_qlog_cc_state(&mut self, now: Instant)
+        -> Option<&'static str>;
+
     fn send_quantum(&self) -> usize;
 
     fn get_next_release_time(&self) -> ReleaseDecision;
@@ -279,8 +290,29 @@ impl Recovery {
         }
     }
 
+    #[cfg(feature = "qlog")]
+    pub fn maybe_qlog(
+        &mut self, qlog: &mut qlog::streamer::QlogStreamer, now: Instant,
+    ) {
+        if let Some(ev_data) = self.get_updated_qlog_event_data() {
+            qlog.add_event_data_with_instant(ev_data, now).ok();
+        }
+
+        if let Some(cc_state) = self.get_updated_qlog_cc_state(now) {
+            let ev_data = EventData::CongestionStateUpdated(
+                qlog::events::quic::CongestionStateUpdated {
+                    old: None,
+                    new: cc_state.to_string(),
+                    trigger: None,
+                },
+            );
+
+            qlog.add_event_data_with_instant(ev_data, now).ok();
+        }
+    }
+
     #[cfg(test)]
-    pub fn new(config: &crate::Config) -> Self {
+    pub fn new(config: &Config) -> Self {
         Self::new_with_config(&RecoveryConfig::from_config(config))
     }
 }
@@ -632,16 +664,15 @@ pub enum StartupExitReason {
 mod tests {
     use super::*;
     use crate::packet;
-    use crate::ranges;
     use crate::recovery::congestion::PACING_MULTIPLIER;
-    use crate::testing;
+    use crate::test_utils;
     use crate::CongestionControlAlgorithm;
     use rstest::rstest;
     use smallvec::smallvec;
     use std::str::FromStr;
 
     fn recovery_for_alg(algo: CongestionControlAlgorithm) -> Recovery {
-        let mut cfg = crate::Config::new(crate::PROTOCOL_VERSION).unwrap();
+        let mut cfg = Config::new(crate::PROTOCOL_VERSION).unwrap();
         cfg.set_cc_algorithm(algo);
         Recovery::new(&cfg)
     }
@@ -691,7 +722,7 @@ mod tests {
         #[values("reno", "cubic", "bbr", "bbr2", "bbr2_gcongestion")]
         cc_algorithm_name: &str,
     ) {
-        let mut cfg = crate::Config::new(crate::PROTOCOL_VERSION).unwrap();
+        let mut cfg = Config::new(crate::PROTOCOL_VERSION).unwrap();
         assert_eq!(cfg.set_cc_algorithm_name(cc_algorithm_name), Ok(()));
 
         let mut r = Recovery::new(&cfg);
@@ -827,7 +858,7 @@ mod tests {
         now += Duration::from_millis(10);
 
         // Only the first 2 packets are acked.
-        let mut acked = ranges::RangeSet::default();
+        let mut acked = RangeSet::default();
         acked.insert(0..2);
 
         assert_eq!(
@@ -928,7 +959,7 @@ mod tests {
         now += Duration::from_millis(10);
 
         // PTO packets are acked.
-        let mut acked = ranges::RangeSet::default();
+        let mut acked = RangeSet::default();
         acked.insert(4..6);
 
         assert_eq!(
@@ -978,7 +1009,7 @@ mod tests {
         #[values("reno", "cubic", "bbr", "bbr2", "bbr2_gcongestion")]
         cc_algorithm_name: &str,
     ) {
-        let mut cfg = crate::Config::new(crate::PROTOCOL_VERSION).unwrap();
+        let mut cfg = Config::new(crate::PROTOCOL_VERSION).unwrap();
         assert_eq!(cfg.set_cc_algorithm_name(cc_algorithm_name), Ok(()));
 
         let mut r = Recovery::new(&cfg);
@@ -1112,7 +1143,7 @@ mod tests {
         now += Duration::from_millis(10);
 
         // Only the first 2 packets and the last one are acked.
-        let mut acked = ranges::RangeSet::default();
+        let mut acked = RangeSet::default();
         acked.insert(0..2);
         acked.insert(3..4);
 
@@ -1175,7 +1206,7 @@ mod tests {
         #[values("reno", "cubic", "bbr", "bbr2", "bbr2_gcongestion")]
         cc_algorithm_name: &str,
     ) {
-        let mut cfg = crate::Config::new(crate::PROTOCOL_VERSION).unwrap();
+        let mut cfg = Config::new(crate::PROTOCOL_VERSION).unwrap();
         assert_eq!(cfg.set_cc_algorithm_name(cc_algorithm_name), Ok(()));
 
         let mut r = Recovery::new(&cfg);
@@ -1309,7 +1340,7 @@ mod tests {
         now += Duration::from_millis(10);
 
         // ACKs are reordered.
-        let mut acked = ranges::RangeSet::default();
+        let mut acked = RangeSet::default();
         acked.insert(2..4);
 
         assert_eq!(
@@ -1333,7 +1364,7 @@ mod tests {
 
         now += Duration::from_millis(10);
 
-        let mut acked = ranges::RangeSet::default();
+        let mut acked = RangeSet::default();
         acked.insert(0..2);
 
         assert_eq!(r.pkt_thresh(), INITIAL_PACKET_THRESHOLD);
@@ -1390,7 +1421,7 @@ mod tests {
         #[values("reno", "cubic", "bbr", "bbr2", "bbr2_gcongestion")]
         cc_algorithm_name: &str,
     ) {
-        let mut cfg = crate::Config::new(crate::PROTOCOL_VERSION).unwrap();
+        let mut cfg = Config::new(crate::PROTOCOL_VERSION).unwrap();
         assert_eq!(cfg.set_cc_algorithm_name(cc_algorithm_name), Ok(()));
 
         let mut r = Recovery::new(&cfg);
@@ -1447,7 +1478,7 @@ mod tests {
         // Wait 50ms for ACK.
         now += Duration::from_millis(50);
 
-        let mut acked = ranges::RangeSet::default();
+        let mut acked = RangeSet::default();
         acked.insert(0..10);
 
         assert_eq!(
@@ -1638,11 +1669,113 @@ mod tests {
         assert_eq!(r.startup_exit(), None);
     }
 
+    #[rstest]
+    // initial_cwnd / first_rtt == initial_pacing_rate.  Pacing is 1.0 * bw before
+    // and after.
+    #[case::bw_estimate_equal_after_first_rtt(1.0, 1.0)]
+    // initial_cwnd / first_rtt < initial_pacing_rate.  Pacing decreases from 2 *
+    // bw to 1.0 * bw.
+    #[case::bw_estimate_decrease_after_first_rtt(2.0, 1.0)]
+    // initial_cwnd / first_rtt > initial_pacing_rate from 0.5 * bw to 1.0 * bw.
+    // Initial pacing remains 0.5 * bw because the initial_pacing_rate parameter
+    // is used an upper bound for the pacing rate after the first RTT.
+    // Pacing rate after the first ACK should be:
+    // min(initial_pacing_rate_bytes_per_second, init_cwnd / first_rtt)
+    #[case::bw_estimate_increase_after_first_rtt(0.5, 0.5)]
+    #[cfg(feature = "internal")]
+    fn initial_pacing_rate_override(
+        #[case] initial_multipler: f64, #[case] expected_multiplier: f64,
+    ) {
+        let rtt = Duration::from_millis(50);
+        let bw = Bandwidth::from_bytes_and_time_delta(12000, rtt);
+        let initial_pacing_rate_hint = bw * initial_multipler;
+        let expected_pacing_with_rtt_measurement = bw * expected_multiplier;
+
+        let cc_algorithm_name = "bbr2_gcongestion";
+        let mut cfg = Config::new(crate::PROTOCOL_VERSION).unwrap();
+        assert_eq!(cfg.set_cc_algorithm_name(cc_algorithm_name), Ok(()));
+        cfg.set_custom_bbr_params(BbrParams {
+            initial_pacing_rate_bytes_per_second: Some(
+                initial_pacing_rate_hint.to_bytes_per_second(),
+            ),
+            ..Default::default()
+        });
+
+        let mut r = Recovery::new(&cfg);
+
+        let mut now = Instant::now();
+
+        assert_eq!(r.sent_packets_len(packet::Epoch::Application), 0);
+
+        // send some packets.
+        for i in 0..2 {
+            let p = test_utils::helper_packet_sent(i, now, 1200);
+            r.on_packet_sent(
+                p,
+                packet::Epoch::Application,
+                HandshakeStatus::default(),
+                now,
+                "",
+            );
+        }
+
+        assert_eq!(r.sent_packets_len(packet::Epoch::Application), 2);
+        assert_eq!(r.bytes_in_flight(), 2400);
+        assert_eq!(r.bytes_in_flight_duration(), Duration::ZERO);
+
+        // Initial pacing rate matches the override value.
+        assert_eq!(
+            r.pacing_rate(),
+            initial_pacing_rate_hint.to_bytes_per_second()
+        );
+        assert_eq!(r.get_packet_send_time(now), now);
+
+        assert_eq!(r.cwnd(), 12000);
+        assert_eq!(r.cwnd_available(), 9600);
+
+        // Wait 1 rtt for ACK.
+        now += rtt;
+
+        let mut acked = RangeSet::default();
+        acked.insert(0..2);
+
+        assert_eq!(
+            r.on_ack_received(
+                &acked,
+                10,
+                packet::Epoch::Application,
+                HandshakeStatus::default(),
+                now,
+                None,
+                "",
+            )
+            .unwrap(),
+            OnAckReceivedOutcome {
+                lost_packets: 0,
+                lost_bytes: 0,
+                acked_bytes: 2400,
+                spurious_losses: 0,
+            }
+        );
+
+        assert_eq!(r.sent_packets_len(packet::Epoch::Application), 0);
+        assert_eq!(r.bytes_in_flight(), 0);
+        assert_eq!(r.bytes_in_flight_duration(), rtt);
+        assert_eq!(r.rtt(), rtt);
+
+        // Pacing rate is recalculated based on initial cwnd when the
+        // first RTT estimate is available.
+        assert_eq!(
+            r.pacing_rate(),
+            expected_pacing_with_rtt_measurement.to_bytes_per_second()
+        );
+    }
+
     #[rstest]
     fn validate_ack_range_on_ack_received(
         #[values("cubic", "bbr2", "bbr2_gcongestion")] cc_algorithm_name: &str,
     ) {
-        let mut cfg = crate::Config::new(crate::PROTOCOL_VERSION).unwrap();
+        let mut cfg = Config::new(crate::PROTOCOL_VERSION).unwrap();
         cfg.set_cc_algorithm_name(cc_algorithm_name).unwrap();
 
         let epoch = packet::Epoch::Application;
@@ -1654,7 +1787,7 @@ mod tests {
         let pkt_size = 1000;
         let pkt_count = 4;
         for pkt_num in 0..pkt_count {
-            let sent = crate::testing::helper_packet_sent(pkt_num, now, pkt_size);
+            let sent = test_utils::helper_packet_sent(pkt_num, now, pkt_size);
             r.on_packet_sent(sent, epoch, HandshakeStatus::default(), now, "");
         }
         assert_eq!(r.sent_packets_len(epoch), pkt_count as usize);
@@ -1666,7 +1799,7 @@ mod tests {
         now += Duration::from_millis(10);
 
         // ACK 2 packets
-        let mut acked = ranges::RangeSet::default();
+        let mut acked = RangeSet::default();
         acked.insert(0..2);
 
         assert_eq!(
@@ -1695,7 +1828,7 @@ mod tests {
         assert_eq!(r.largest_sent_pkt_num_on_path(epoch).unwrap(), 3);
 
         // ACK large range
-        let mut acked = ranges::RangeSet::default();
+        let mut acked = RangeSet::default();
         acked.insert(0..10);
         assert_eq!(
             r.on_ack_received(
@@ -1727,7 +1860,7 @@ mod tests {
         #[values("reno", "cubic", "bbr", "bbr2", "bbr2_gcongestion")]
         cc_algorithm_name: &str,
     ) {
-        let mut cfg = crate::Config::new(crate::PROTOCOL_VERSION).unwrap();
+        let mut cfg = Config::new(crate::PROTOCOL_VERSION).unwrap();
         assert_eq!(cfg.set_cc_algorithm_name(cc_algorithm_name), Ok(()));
 
         let mut r = Recovery::new(&cfg);
@@ -1832,7 +1965,7 @@ mod tests {
         now += Duration::from_millis(10);
 
         // Only the first  packets and the last one are acked.
-        let mut acked = ranges::RangeSet::default();
+        let mut acked = RangeSet::default();
         acked.insert(0..1);
         acked.insert(2..3);
 
@@ -1901,7 +2034,7 @@ mod tests {
     fn congestion_delivery_rate(
         #[values("reno", "cubic", "bbr", "bbr2")] cc_algorithm_name: &str,
     ) {
-        let mut cfg = crate::Config::new(crate::PROTOCOL_VERSION).unwrap();
+        let mut cfg = Config::new(crate::PROTOCOL_VERSION).unwrap();
         assert_eq!(cfg.set_cc_algorithm_name(cc_algorithm_name), Ok(()));
 
         let mut r = Recovery::new(&cfg);
@@ -1913,7 +2046,7 @@ mod tests {
         for pn in 0..10 {
             // Start by sending a few packets.
             let bytes = 1000;
-            let sent = testing::helper_packet_sent(pn, now, bytes);
+            let sent = test_utils::helper_packet_sent(pn, now, bytes);
             r.on_packet_sent(
                 sent,
                 packet::Epoch::Application,
@@ -1927,7 +2060,7 @@ mod tests {
 
         // Ack
         let interval = Duration::from_secs(10);
-        let mut acked = ranges::RangeSet::default();
+        let mut acked = RangeSet::default();
         acked.insert(0..10);
         assert_eq!(
             r.on_ack_received(
@@ -1938,7 +2071,8 @@ mod tests {
                 now + interval,
                 None,
                 "",
-            ).unwrap(),
+            )
+            .unwrap(),
             OnAckReceivedOutcome {
                 lost_packets: 0,
                 lost_bytes: 0,
